#!/bin/bash
#
# IceWarp Server installation package
# (c) IceWarp Ltd. 2008
#
#	All rights reserved.
#
# http://www.icewarp.com 
#
# file: install.sh - install script
# version: 1.5
#

# defaults

USE_COLORS="0"

NEWVERSION="10.0.3 DEB5.0.3"
IMAGENAME="IceWarpServer-image-10.0.3_DEB5.0.3.tar"

# default variables
SERVERINSTALLDIR="/opt/icewarp"
USER=$(id -un)
GROUP=$(id -gn)
UUID=$(id -u)
CHROOT="0"
SERVERCONF="UNKNOWN"
ERRORLOG=~/icewarp-install-error.log
INSTALLLOG=~/icewarp-install.log
UPGRADE="0"
OS_PLATFORM="UNKNOWN"
SERVICE_NAME="icewarp"

# init

SELF=$0
WORKDIR=$(dirname $0)

if  [ "$TERM" = "linux" ] || [ "$TERM" = "xterm" ] || [ "$TERM" = "xterm-color" ] || [ "$TERM" = "screen" ] && [ "$USE_COLORS" = "1" ]
    then
	GOOD=$'\e[32;01m'
	WARN=$'\e[33;01m'
	BAD=$'\e[31;01m'
	NORMAL=$'\e[0m'
	HILITE=$'\e[37;01m'
	BRACKET=$'\e[34;01m'
    else
	GOOD=$'\e[0;32m'
	BAD=$'\e[0;31m'
	NORMAL=$'\e[0m'
	WARN=${NORMAL}
	HILITE=${NORMAL}
	BRACKET=${NORMAL}
fi

# Detection of host OS platform
# Function tryes to detect OS in simple way
# Result of detection is:
# - "UNKNOWN" if platform is not known
# - "RHEL" if OS is Red Hat Enterprise Linux
# TODO:
# add more precise detection
# detect more platforms
detect_os_platform()
{
    if [ -f "/etc/redhat-release" ]
	then
	    OS_PLATFORM="RHEL"
    fi
}

check_server_is_running()
{
	good "Please make sure the IceWarp Mail Server is not running before installing."
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

# Ask question with confirmation
# Parameters:
# $1 - question
# $2 - first choice, it is choosed if user enters only empty string
# $3 - second choice
# Return:
# 0 on error - unknown string entered
# 1 if first choice was selected
# 2 if second choice was selected
ask_with_confirmation()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
	then
	    return 0
    fi
    LOWER_CHOICE_1=$(echo $2 | tr "A-Z" "a-z")
    LOWER_CHOICE_2=$(echo $3 | tr "A-Z" "a-z")
    echo -e -n "${GOOD}**${NORMAL}\t$1 [${HILITE}$2${NORMAL}/$3]: "
    read PARAM
    PARAM=$(echo "$PARAM" | tr "A-Z" "a-z")
    if [ "$PARAM" == "" ] || [ "$PARAM" == "$LOWER_CHOICE_1" ]
	then
	    return 1
    fi
    if [ "$PARAM" == "$LOWER_CHOICE_2" ]
	then
	    return 2
    fi
    return 0
}

preconf()
{
    if [ -f "$SERVERCONF" ]
	then
	    source "$SERVERCONF"
	    [[ "$InstallDir" != "" ]] && SERVERINSTALLDIR=$InstallDir
	    [[ "$User" != "" ]] && USER=$User
	    [[ "$Group" != "" ]] && GROUP=$Group
	    [[ "$Chroot" != "" ]] && CHROOT=$Chroot
	    # hack to fill in version number if empty
	    OLDVERSION=$Version
    fi
}

good()
{
    echo -e "${GOOD}**${NORMAL}\t$*"
}

bad()
{
    echo -e "${BAD}**${NORMAL}\tError: $*"
}

warn()
{
    echo -e "${WARN}**${NORMAL}\tWarning: $*"
}

hilite()
{
    echo -e "${GOOD}**${HILITE}\t$*${NORMAL}"
}

getparam()
{
    echo -e -n "${GOOD}**${NORMAL}\t$1 [$2]${NORMAL}: "
    read PARAM
}

# Function extracts install image to destination directory and to 
# directories setuped in load balanced mode
# It recovers missing configuration files
# Parameters:
# $1 - tar image name
# $2 - destination directory where to extract files
extract_image()
{
    good "Please wait..."

    tar -xf "$1" -C "$2"
    if [ $? -ne 0 ]; then
	bad "Can not extract image $1"
	exit 1
    fi

    # check if configuration files exist
    # this will create all configuration files in case of new installation
    # and add missing files in case of upgrade
    if [ "$UPGRADE" == "0" ]
	then
	    # copy all configuration files
	    cp -f "$2/config/default/cert.pem" "$2/config/cert.pem"
	    cp -f "$2/config/default/webserver.dat" "$2/config/webserver.dat"

	    cp -f "$2/calendar/default/calendar.dat" "$2/calendar/calendar.dat"
	    cp -f "$2/calendar/default/db/groupware.db" "$2/calendar/groupware.db"

	    cp -f "$2/spam/default/db/antispam.db" "$2/spam/antispam.db"
	    cp -f "$2/spam/default/rules/local.cf" "$2/spam/rules/local.cf"
	else
	    # Read destination directories for calendar, config and spam
	    # Already installed server can be configured to use custom directories
	    DST_DIR_CALENDAR=$("$2/tool.sh" display system C_CalendarPath)
	    DST_DIR_CALENDAR=${DST_DIR_CALENDAR##"C_CalendarPath: "}

	    DST_DIR_CONFIG=$("$2/tool.sh" display system C_ConfigPath)
	    DST_DIR_CONFIG=${DST_DIR_CONFIG##"C_ConfigPath: "}

	    DST_DIR_SPAM=$("$2/tool.sh" display system C_SpamPath)
	    DST_DIR_SPAM=${DST_DIR_SPAM##"C_SpamPath: "}

	    # extract calendar, config and spam directories into temporary folder
	    DIR_TEMP=$(mktemp -d)

	    tar -xf "$1" -C "$DIR_TEMP" calendar
	    if [ $? -ne 0 ]; then
		bad "Can not extract calendar files from image"
		exit 1
	    fi
	    tar -xf "$1" -C "$DIR_TEMP" config
	    if [ $? -ne 0 ]; then
		bad "Can not extract config files from image"
		exit 1
	    fi
	    tar -xf "$1" -C "$DIR_TEMP" spam
	    if [ $? -ne 0 ]; then
		bad "Can not extract spam files from image"
		exit 1
	    fi

	    # copy files from temp to destination
	    cp -fr "${DIR_TEMP}/calendar/"* "$DST_DIR_CALENDAR"
	    if [ $? -ne 0 ]; then
		bad "Can not copy calendar files"
		exit 1
	    fi
	    cp -fr "${DIR_TEMP}/config/"* "$DST_DIR_CONFIG"
	    if [ $? -ne 0 ]; then
		bad "Can not copy config files"
		exit 1
	    fi
	    cp -fr "${DIR_TEMP}/spam/"* "$DST_DIR_SPAM"
	    if [ $? -ne 0 ]; then
		bad "Can not copy spam files"
		exit 1
	    fi

	    # remove temoporary directory
	    rm -rf "$DIR_TEMP"

	    # test if file exist, if not copy default ones

	    #
	    # Files in calendar directory
	    #

	    # calendar/calendar.dat
	    FNAME_SRC="${DST_DIR_CALENDAR}/default/calendar.dat"
	    FNAME_DST="${DST_DIR_CALENDAR}/calendar.dat"
	    if [ ! -f "$FNAME_DST" ]
		then
		    good "Configuration file ${FNAME_DST} des not exist, copying from default."
		    cp -f "$FNAME_SRC" "$FNAME_DST"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi
	    # calendar/groupware.db
	    FNAME_SRC="${DST_DIR_CALENDAR}/default/db/groupware.db"
	    FNAME_DST="${DST_DIR_CALENDAR}/groupware.db"
	    if [ ! -f "$FNAME_DST" ]
		then
		    good "Configuration file ${FNAME_DST} des not exist, copying from default."
		    cp -f "$FNAME_SRC" "$FNAME_DST"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi

	    #
	    # Files in config directory
	    #

	    # config/cert.pem
	    FNAME_SRC="${DST_DIR_CONFIG}/default/cert.pem"
	    FNAME_DST="${DST_DIR_CONFIG}/cert.pem"
	    if [ ! -f "${FNAME_DST}" ]
		then
		    good "Configuration file ${FNAME_DST} does not exist, copying from default."
		    cp -f "${FNAME_SRC}" "${FNAME_DST}"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi
	    # config/webserver.dat
	    FNAME_SRC="${DST_DIR_CONFIG}/default/webserver.dat"
	    FNAME_DST="${DST_DIR_CONFIG}/webserver.dat"
	    if [ ! -f "${FNAME_DST}" ]
		then
		    good "Configuration file ${FNAME_DST} does not exist, copying from default."
		    cp -f "$FNAME_SRC" "$FNAME_DST"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi

	    #
	    # Files in spam directory
	    #

	    # spam/antispam.db
	    FNAME_SRC="${DST_DIR_SPAM}/default/db/antispam.db"
	    FNAME_DST="${DST_DIR_SPAM}/antispam.db"
	    if [ ! -f "$FNAME_DST" ]
		then
		    good "Configuration file ${FNAME_DST} des not exist, copying from default."
		    cp -f "$FNAME_SRC" "$FNAME_DST"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi
	    # spam/rules/local.cf
	    FNAME_SRC="${DST_DIR_SPAM}/default/rules/local.cf"
	    FNAME_DST="${DST_DIR_SPAM}/rules/local.cf"
	    if [ ! -f "$FNAME_DST" ]
		then
		    good "Configuration file ${FNAME_DST} des not exist, copying from default."
		    cp -f "$FNAME_SRC" "$FNAME_DST"
		    if [ $? -ne 0 ]; then
			bad "Can not copy ${FNAME_SRC} to ${FNAME_DST}"
			exit 1
		    fi
	    fi
    fi
    if [ -e info.xml ]
	then
	    mkdir -p "$2/xml" 2> /dev/null
	    cp -f info.xml "$2/xml/"
    fi
}

ask_questions()
{
    getparam "Installation prefix" "$SERVERINSTALLDIR"
    [[ "$PARAM" != "" ]] && SERVERINSTALLDIR=$PARAM

    # check for "new installation" or "upgrade" 
    # check if installation directory exist
    if [ -d "$SERVERINSTALLDIR" ]; then
		UPGRADE="2"
    fi
    # check if installation file exist but it is not directory
    if [ "$UPGRADE" == "0" -a -e "$SERVERINSTALLDIR" ];	then
	    bad "File ${SERVERINSTALLDIR} exists but it is not directory."
	    exit 1
    fi

    # check if merak.conf or icewarp.conf configuration file exist
    SERVERCONF="${SERVERINSTALLDIR}/config/merak.conf"
    if [ -f "$SERVERCONF" ]; then
    	UPGRADE="1"
    else
    	SERVERCONF="${SERVERINSTALLDIR}/config/icewarp.conf"
   		if [ -f "$SERVERCONF" ]; then
    		UPGRADE="1"
   		fi
    fi

    # if UPGRADE is 2 then SERVERINSTALLDIR exist but configuration file does not exist
    if [ "$UPGRADE" == "2" ]; then
		good ""
		good "Directory ${SERVERINSTALLDIR} exists but"
		good "server configuration file does not exist."
		good ""
		good "Please select full installation or upgrade."
		good "WARNING: directory ${SERVERINSTALLDIR} will be DELETED in full installation!"
		good ""
		ask_with_confirmation "Upgrade or Install?" "U" "i"
		if [ $? -eq 1 ]
			then
				UPGRADE="1"
			else
				UPGRADE="0"
		fi
    fi

    if  [ "$UPGRADE" == "1" ]
		then
	    	# read existing configuration file
	    	preconf
	    	echo "${GOOD}** Performing upgrade"
		else
	    	echo "${GOOD}** Performing install"

			getparam "Run services as user" "$USER"
			[[ "$PARAM" != "" ]] && USER=$PARAM

			getparam "Run services as group" "$GROUP"
			[[ "$PARAM" != "" ]] && GROUP=$PARAM
    fi
}

confirm_params()
{
    good ""
    good "Please check entered informations before continuing:"
    good ""
    if [ "$UPGRADE" == "1" ]
	then
	    good "Installation prefix:\t\t${SERVERINSTALLDIR} (upgrading)"
	    else
		good "Installation prefix:\t\t${SERVERINSTALLDIR} (directory will be created)"
    fi

    good "Services will run as user:\t${USER}"
    good "Services will run as group:\t${GROUP}"

    if [[ "$CHROOT" == "1" ]]
	then
	    good "Services will run in chroot environment"
    fi

    good ""
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

check_entered_params()
{
    # check if entered user and group exist
    grep -q "^$USER:" /etc/passwd
    if [ $? -ne 0 ]
        then
	    bad "User ${USER} doesn't exist."
	    exit 1
    fi
    grep -q "^$GROUP:" /etc/group
    if [ $? -ne 0 ]
	then
	    bad "Group ${GROUP} doesn't exist."
	    exit 1
    fi
    id "$USER" 2>&1 | grep -q "(${GROUP})"
    if [ $? -ne 0 ]
	then
	    bad "User ${USER} is not a member of group ${GROUP}!"
	    exit 1
    fi
    # simple check if have permissions to install
    UUID_DST=$(id -u "$USER")
    if [ $UUID -ne 0 ]
	then
	    if [ $UUID -ne $UUID_DST ]
		then
		    bad "Permission denied to install for user ${USER}"
		    exit 1
	    fi
    fi
}

install_package()
{
    if [ "$UPGRADE" == "0" ]
	then
	    good "Creating ${SERVERINSTALLDIR} directory"

	    if [ "$SERVERINSTALLDIR" == "/" ]
	    then
		bad "Rejecting directory removal: ${SERVERINSTALLDIR}"
		exit 1
	    fi
	    rm -rf "$SERVERINSTALLDIR"
	    mkdir -p "$SERVERINSTALLDIR"
	    if [ $? -ne 0 ]
		then
		    bad "Failed! Check user permissions"
		    exit 1
	    fi
	else
	    # remove files which are not in use anymore
	    # but not changed by extracting from tar

	    # scripts
	    rm -rf "${SERVERINSTALLDIR}/scripts"
	    rm -f "${SERVERINSTALLDIR}/startd"
	    rm -f "${SERVERINSTALLDIR}/stopd"
	    rm -f "${SERVERINSTALLDIR}/restartd"
	    rm -f "${SERVERINSTALLDIR}/merakd.sh"
	    # binaries
	    rm -f "${SERVERINSTALLDIR}/merakd"
	    # merak.conf
	    rm -f "${SERVERINSTALLDIR}/config/merak.conf"
    fi
    good "Extracting distribution"
    extract_image "${WORKDIR}/${IMAGENAME}" "$SERVERINSTALLDIR"
    if [ $? -ne 0 ]
	then
	    bad "Failed, check user permissions and available disk space. 70MB disk space is needed."
	    exit 1
    fi
}

configure_package()
{
#	if ( touch "$SERVERCONF" )
#	    then 
	cp "$SERVERCONF" "${SERVERCONF}.bak" &> /dev/null
	DATE=`date`
	echo "## IceWarp Server configuration file
## this file was generated by IceWarp installer on ${DATE}
## InstallDir=/path/to/server
## Server installation prefix, absolute path, no useful default

InstallDir=\"${SERVERINSTALLDIR}\"

## Version=number
## IceWarp Server version, for reference only

Version=\"${NEWVERSION}\"
 
## User=username
## User to run services as, default=root

User=\"${USER}\"
 
## Group=group
## Group to run services as, default=root

Group=\"${GROUP}\"

## Capabilities
## capabilities that are set at startup, see
## man cap_from_text
## for more info
## ADVANCED USERS ONLY!

Capabilities=\"\"

AVENGINE=\"${AVENGINE}\"

# Previous and new version number - used for upgrade purposes

OLDVERSION=\"${OLDVERSION}\"" > "$SERVERCONF"

#	    else
#		warn "File ${SERVERCONF} is not writable, skipping it."
#	    fi

#	    if ( touch ${SERVERINSTALLDIR}/php/php.ini )
#		then

# Print settings to php.ini
echo "
; xcache must be loaded as zend_extension and before Zend optimizer
; otherwise it can cause php crash
zend_extension = \"${SERVERINSTALLDIR}/php/ext/xcache.so\"
" >> "${SERVERINSTALLDIR}/php/php.ini"

echo "
[Zend]
zend_extension_manager.optimizer = \"${SERVERINSTALLDIR}/php/ext/Optimizer-3.3.3\"
zend_extension_manager.optimizer_ts = \"${SERVERINSTALLDIR}/php/ext/Optimizer_TS-3.3.3\"
zend_optimizer.version = 3.3.3
zend_extension = \"${SERVERINSTALLDIR}/php/ext/ZendExtensionManager.so\"
zend_extension_ts = \"${SERVERINSTALLDIR}/php/ext/ZendExtensionManager_TS.so\"
" >> "${SERVERINSTALLDIR}/php/php.ini"

echo "
[IceWarp]
icewarp_sharedlib_path = \"${SERVERINSTALLDIR}/html/_shared/\"
extension_dir = \"${SERVERINSTALLDIR}/php/ext/\"
error_log = \"${SERVERINSTALLDIR}/logs/phperror.log\"
open_basedir_ignore = \"${SERVERINSTALLDIR}/\"
upload_tmp_dir = \"${SERVERINSTALLDIR}/php/tmp/\"
session.save_path = \"${SERVERINSTALLDIR}/php/tmp/\"
sendmail_path = \"${SERVERINSTALLDIR}/sendmail\"
" >> "${SERVERINSTALLDIR}/php/php.ini"

#		else
#		    warn "File ${SERVERINSTALLDIR}/php/php.ini is not writable, skipping it."
#		    warn "Web interface may misbehave."
#	    fi
}

run_postconf()
{
    # install links to .so libraries
    
    # add/remove merakd as service
    # this option is available only for root
    SERVICE_INSTALLED="0"
    SERVICE_STARTED="0"
    if [ "$UUID" -eq 0 -a "$OS_PLATFORM" == "RHEL" ]
	then
		# check if merak service is installed
		good "Checking for previous 'merak' service installation ..."
	    service merak status > /dev/null
	    if [ $? -eq 0 ]; then
	    	good "Service 'merak' was found installed in system"
	    	good "which is now obsolete and renamed to 'icewarp' service."
	    	ask_with_confirmation "Delete service 'merak'?" "Y" "n"
	    	if [ $? -eq 1 ]; then
			    good "Removing 'merak' system service ..."
			    chkconfig --del merak > /dev/null
			    rm -f "/etc/init.d/merak"
	    	fi
	    fi

	    # check if system service is installed
	    good "Checking if IceWarp Server is added as system service ..."
	    good "Note: System service can be reinstalled"
	    good "      by removing already installed service."
	    good ""
	    service "$SERVICE_NAME" status > /dev/null
	    if [ $? -eq 0 ]
		then
		    SERVICE_INSTALLED="1"
		    good "IceWarp Server is already added as system service."
		    ask_with_confirmation "Delete service?" "Y" "n"
		    if [ $? -eq 1 ]
			then
			    # remove merakd from service manager
			    good "Removing IceWarp Server system service ..."
			    chkconfig --del "$SERVICE_NAME" > /dev/null
			    rm -f "/etc/init.d/${SERVICE_NAME}"
			    SERVICE_INSTALLED="0"
		    fi
	    fi

	    # ask and add icewarpd as service
	    if [ "$SERVICE_INSTALLED" == "0" ]
		then
		    ask_with_confirmation "Do you want to add IceWarp Server as a system service?" "Y" "n"
		    if [ $? -eq 1 ]
			then
			    good "Adding IceWarp Server as system service"
			    # create new init script
			    sed "s|%%INSTALL_DIR_VALUE%%|${SERVERINSTALLDIR}|" "${SERVERINSTALLDIR}/scripts/setup/${SERVICE_NAME}.init" > "/etc/init.d/${SERVICE_NAME}"
			    if [ $? -ne 0 ]
				then
				    bad "Cannot create init file."
				    exit 1
			    fi
			    chmod a+x "/etc/init.d/${SERVICE_NAME}" > /dev/null
			    if [ $? -ne 0 ]
				then
				    bad "Cannot change permissions on init file."
				    exit 1
			    fi
			    SERVICE_INSTALLED="1"
			else
			    good "IceWarp Server will not be added as system service."
		    fi
	    fi

	    # ask and add or remove service to autostart at boot time
	    if [ "$SERVICE_INSTALLED" == "1" ]
		then
		    ask_with_confirmation "Do you want IceWarp Server to start automatically on reboot?" "Y" "n"
		    if [ $? -eq 1 ]; then
   		    	good "Making service as autostart"
	    		chkconfig --del "$SERVICE_NAME" > /dev/null
			    chkconfig --add "$SERVICE_NAME" > /dev/null
			    if [ $? -ne 0 ]; then
				    bad "Cannot add service to autostart."
				    exit 1
			    fi
			else
			    good "IceWarp Server will not start automatically on reboot."
			    chkconfig --del "$SERVICE_NAME" > /dev/null
		fi
	    fi
    fi

    # run upgrade tool if upgrading
    if [ "$UPGRADE" == "1" ]
	then
	    good ""
	    good "It is possible now to upgrade data to the current version."
	    good "To do this, all services have to be started."
	    good "Note: You can run upgrade process later by command"
	    good "      in ${SERVERINSTALLDIR}: ./tool upgrade _old_version_"
	    good ""
	    ask_with_confirmation "Start services and perform the data upgrade now?" "Y" "n"
	    if [ $? -eq 1 ]
		then
		    good "Starting IceWarp Server"
		    "${SERVERINSTALLDIR}/icewarpd.sh" --start > /dev/null
		    if [ $? -ne 0 ]
			then
			    bad "Cannot start IceWarp Server"
			    exit 1
			else
			    MERAK_STARTED="1"
		    fi
		    "${SERVERINSTALLDIR}/tool.sh" upgrade "$OLD_VERSION" > /dev/null
		    if [ $? -eq 0 ]
			then
			    good "Data upgrade completed successfully"
			else
			    bad "Data upgrade problem!"
		    fi
	    fi
    fi
}

update_configuration_files()
{
    # do not upgrade configuration files when installing
    if [ "$UPGRADE" == "0" ]
	then
	    return 0
    fi

    good ""
    good "Upgrading configuration files:"

    # update webserver.dat
    CONFIG_FILE="${SERVERINSTALLDIR}/config/webserver.dat"
    good "$CONFIG_FILE"
    sed "s|phpd<|phpd.sh<|" "$CONFIG_FILE" > "${CONFIG_FILE}_"
    rm -f "$CONFIG_FILE"
    mv "${CONFIG_FILE}_" "$CONFIG_FILE"

    return 0
}

create_uninstall_info()
{
    # create list of all files in tar
    tar -tf "${WORKDIR}/${IMAGENAME}" > "${SERVERINSTALLDIR}/scripts/setup/content.txt" 2> /dev/null

    # split tar content to directories and files
    INSTALLED_DIRS=$(cat "${SERVERINSTALLDIR}/scripts/setup/content.txt" | grep ".*\/$")
    INSTALLED_FILES=$(cat "${SERVERINSTALLDIR}/scripts/setup/content.txt" | grep ".*[^\/]$")

    # create file with installed directories
    rm -f "${SERVERINSTALLDIR}/scripts/setup/installed_dirs.txt"
    touch "${SERVERINSTALLDIR}/scripts/setup/installed_dirs.txt"
    for LINE in $INSTALLED_DIRS
    do
	echo $LINE >> "${SERVERINSTALLDIR}/scripts/setup/installed_dirs.txt"
    done

    # create file with installed files
    # do not count configuration files which comes with installation
    rm -f "${SERVERINSTALLDIR}/scripts/setup/installed_files.txt"
    touch "${SERVERINSTALLDIR}/scripts/setup/installed_files.txt"
    for LINE in $INSTALLED_FILES
    do
	if  [ "$LINE" != "config/cert.pem" ] &&
	    [ "$LINE" != "config/webserver.dat" ] &&
	    [ "$LINE" != "spam/antispam.db" ] &&
	    [ "$LINE" != "spam/rules/local.cf" ] &&
	    [ "$LINE" != "calendar/groupware.db" ]
	     then
		echo $LINE >> "${SERVERINSTALLDIR}/scripts/setup/installed_files.txt"
	fi
    done

    # add installed_dirs.txt file
    echo "scripts/setup/installed_dirs.txt" >> "${SERVERINSTALLDIR}/scripts/setup/installed_files.txt"
    # add installed_files.txt file
    echo "scripts/setup/installed_files.txt" >> "${SERVERINSTALLDIR}/scripts/setup/installed_files.txt"

    # remove content file
    rm -f "${SERVERINSTALLDIR}/scripts/setup/content.txt"
}

change_user_owner()
{
	# All files are installed OK, change permissions
    good "Changing permissions"
    chown -R "${USER}:${GROUP}" "$SERVERINSTALLDIR" || bad "Failed! Check user permissions on ${SERVERINSTALLDIR}"
    chmod +s "${SERVERINSTALLDIR}/tool.sh"
    chmod +s "${SERVERINSTALLDIR}/icewarpd.sh"
}

run_wizard()
{
    # run wizard if not upgrading
    if [ "$UPGRADE" == "0" ]
	then
	    good ""
	    ask_with_confirmation "Do you want to run IceWarp Server Wizard now?" "Y" "n"
	    if [ $? -eq 1 ]
		then
		    if [ "$SERVICE_STARTED" == "0" ]
			then
			    good "Starting IceWarp Server"
			    "${SERVERINSTALLDIR}/icewarpd.sh" --start > /dev/null
			    if [ $? -ne 0 ]
				then
				    bad "Cannot start IceWarp Server"
				    exit 1
			    fi
		    fi
		    "${SERVERINSTALLDIR}/wizard.sh"
		else
		    good "You can re-run wizard at a later time with script: ${SERVERINSTALLDIR}/wizard.sh"
	    fi
    fi
}

copyright()
{
    good ""
    good "IceWarp Server installer"
    good "(c) 1999-2008 IceWarp Ltd. "
    good ""
}

rejoice()
{
    good ""
    good "IceWarp Server was successfully installed or upgraded."
    good ""
    good "Installer log is available in ${INSTALLLOG}"
    good "Installer error log is available in ${ERRORLOG}"
}

accept_license()
{
    less "${WORKDIR}/LICENSE"
    echo ""
    good "You must accept this license agreement if you want to continue."
    good "Press ENTER to accept license or CTRL+C to quit"
    read -s
    echo ""
}

getscriptparams()
{
    case $1 in
	"-h"|"--help")
	    copyright
	    good ""
	    good "Usage: $0"
	    good ""
	    good "This script provides functionality for installing or upgrading IceWarp Server."
	    good "Default action is to install but if the following conditions are met,"
	    good "only upgrade will be performed:"
	    good "  1. entered destination directory exists"
	    good "  2. file _ENTERED_DESTINATION_DIRECTORY_/config/merak.conf or"
	    good "     file _ENTERED_DESTINATION_DIRECTORY_/config/icewarp.conf exists"
	    good ""
	    good "Install mode extracts all files and creates configuration files"
	    good "from default configuration file directories."
	    good ""
	    good "Upgrade mode:"
	    good "  1. replaces all files which are not configuration files"
	    good "     (i.e. files not modified by running server)"
	    good "  2. recreates default configuration files that do not exists:"
	    good "       ./config/cert.pem"
	    good "       ./config/webserver.dat"
	    good "       ./spam/antispam.dat"
	    good "       ./spam/rules/local.cf"
	    good "       ./calendar/groupware.dat"
	    good ""
	    good "[END]"
	;;
	*)
	    accept_license
	    check_server_is_running
	    good "Installation/Upgrade mode entered"
	    good ""
	    ask_questions
	    confirm_params
	    check_entered_params
	    install_package
	    configure_package
	    update_configuration_files
	    run_postconf
	    create_uninstall_info
	    change_user_owner
	    run_wizard
	    rejoice
	;;
    esac
}

# here we go, let's have fun!

preconf

DATE=$(date)

touch "$INSTALLLOG"
touch "$ERRORLOG"

echo "INSTALLATION STARTED ON ${DATE}" > "$INSTALLLOG"
echo "INSTALLATION STARTED ON ${DATE}" > "$ERRORLOG"
detect_os_platform
getscriptparams $* 2>> "$ERRORLOG" | tee -a "$INSTALLLOG"
echo "INSTALLATION FINISHED ON ${DATE}" >> "$INSTALLLOG"
echo "INSTALLATION FINISHED ON ${DATE}" >> "$ERRORLOG"

exit 0
